import {actionTypes} from '@servicenow/ui-core';

const {COMPONENT_CONNECTED, COMPONENT_DISCONNECTED} = actionTypes;
const BEHAVIOR_NAME = 'keyBindings';

const elementListeners = new Map();

/**
 * This behavior adds keyboard accessibility functionality for Seismic
 * components. You can specify which key events trigger which handlers, and what
 * to do inside of each handler when it detects those key events.
 *
 * After installation import the behavior at the top of your component JS file
 * and register it in your component behaviors list:
 *
 * ```js
 * import keyBindingBehavior from '@servicenow/behavior-key-binding';
 *
 * createCustomElement('sn-example', {
 *   // ...
 *   behaviors: [
 *     {
 *       behavior: keyBindingBehavior,
 *       options: {
 *         keyBindings: {
 *           // accepts a single key value (string) or array of key values (string[])
 *           onEnterKeyPress: 'Enter',
 *           onArrowKeyPress: ['ArrowUp','ArrowDown','ArrowLeft','ArrowRight']
 *         },
 * 		   handlers: {
 *           onArrowKeyPress: function(host, event, dispatch, updateState) {
 *             // 'host' is the component that registered this behavior
 *             // 'event' is the native keyboardEvent that was fired
 *             // 'dispatch' can be used for setting properties or firing other actions
 *             // 'updateState' can be used for updating Seismic internal state
 *           },
 *           onEnterKeyPress: function(host, event, dispatch, updateState) {
 *             // since `event` is the native keyboardEvent, you can use any properties or
 * 			   // methods from the API you wish, e.g. `event.shiftKey` or `event.preventDefault()`
 *           }
 *         }
 *       }
 *     }
 *   ]
 * });
 * ```
 *
 * Please refer to the full list of valid `key` values
 * [on the MDN KeyboardEvents page](https://developer.mozilla.org/en-US/docs/Web/API/KeyboardEvent/key/Key_Values).
 * Note: the behavior currently fires the handler function on `keydown`
 * **only** - a future enhancement may enable you to specify `keypress` or
 * `keyup` instead. The behavior configuration options only supports detection
 * of single key events, but your handlers can be configured to detect the
 * presence of modifier keys or key combinations.
 *
 * @seismicBehavior keyBindingBehavior
 * @summary Adds keyboard accessibility functionality for Seismic components
 */
export default {
	name: BEHAVIOR_NAME,
	actionHandlers: {
		[COMPONENT_CONNECTED]: ({dispatch, updateState, action: {payload}}) => {
			const {host, options} = payload;
			const {keyBindings, handlers} = options;

			const keydownListener = (event) => {
				Object.keys(keyBindings).forEach((fn) => {
					// Supports a simple string (for single key) or array of strings (for multiple keys)
					// e.g. { 'onTab' : 'Tab' }
					// e.g. { 'move' : ['ArrowLeft', 'ArrowRight'] }
					if (
						(Array.isArray(keyBindings[fn]) &&
							keyBindings[fn].includes(event.key)) ||
						(typeof keyBindings[fn] === 'string' &&
							keyBindings[fn] === event.key)
					) {
						handlers[fn].call(host, host, event, dispatch, updateState);
					}
				});
			};

			elementListeners.set(host, keydownListener);
			host.addEventListener('keydown', keydownListener);
		},
		[COMPONENT_DISCONNECTED]: ({action: {payload}}) => {
			const {host} = payload;
			const keydownListener = elementListeners.get(host);
			if (keydownListener) {
				elementListeners.delete(host);
				host.removeEventListener('keydown', keydownListener);
			}
		}
	}
};
